//=======================================================================
/**
 * @file BaseFilter.h
 * @author Priyonto M Rahman
 * @copyright Copyright (c) 2023, Priyonto M Rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================

#ifndef BASEFILTER_H
#define BASEFILTER_H

#include <vector>

/**
 * @ingroup JackExternalClient
 * @class BaseFilter
 * @brief An abstract class that defines a basic audio filter.
 *
 * Derived classes must implement the process_sample() function, which processes an audio sample.
 */
class BaseFilter {
public:
    /// Destructor
    virtual ~BaseFilter();

    /**
     * @brief Pure virtual function to process a buffer of input samples.
     *
     * This function is expected to be overridden in any class that inherits
     * from this base class. The overriding function should take a vector of
     * floating point samples as input, process each sample, and return a new
     * vector containing the processed samples.
     *
     * This method should be used when it's more efficient to process a whole
     * buffer of samples at once, rather than calling a process function on each
     * individual sample.
     *
     * @param inputBuffer A vector of input samples to be processed.
     * @return A vector of output samples that have been processed.
     */
    virtual std::vector<float> process_buffer(std::vector<float>& inputBuffer) = 0;

    /**
     * @brief Process a single sample through the filter.
     *
     * This is a pure virtual function that must be implemented by any class
     * that inherits from this class. This function is called for each sample
     * in the audio stream and processes the sample through the filter.
     *
     * @param sample The sample to be processed. This is expected to be a floating
     *               point value representing the audio sample.
     * @return The processed sample. The processing applied to the sample will
     *         depend on the specifics of the filter that implements this function.
     */
    virtual float process_sample(float sample) = 0;


};

#endif // BASEFILTER_H
