//=======================================================================
/**
 * @file ConfusionMatrixWidget.h
 * @author Priyonto M Rahman
 * @copyright Copyright (c) 2023, Priyonto M Rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================


#ifndef CONFUSIONMATRIXWIDGET_H
#define CONFUSIONMATRIXWIDGET_H

#include <QWidget>
#include <QTableWidget>

/**
 * @ingroup StatisticsUI
 * @class ConfusionMatrixWidget
 * @brief The ConfusionMatrixWidget class provides a custom QWidget to display a confusion matrix.
 *
 * This widget uses a QTableWidget to visually represent a confusion matrix.
 */
class ConfusionMatrixWidget : public QWidget
{
    Q_OBJECT

public:
    /**
     * @brief Construct a new Confusion Matrix Widget object.
     *
     * @param parent The parent QWidget. Default is nullptr.
     */
    explicit ConfusionMatrixWidget(QWidget *parent = nullptr);

    /**
     * @brief Populate the confusion matrix with data.
     *
     * This function populates the confusion matrix based on the provided confusion matrix counts.
     *
     * @param confusionMatrixCounts A map where the key is the actual class, and the value is another map.
     *                              The inner map's key is the predicted class, and its value is the count of instances.
     */
    void populateMatrix(QMap<QString, QMap<QString, int>> &confusionMatrixCounts);

private:
    QTableWidget *matrixTable; ///< A pointer to the QTableWidget that displays the confusion matrix.

    /**
     * @brief Update a specific cell in the confusion matrix.
     *
     * This function updates the cell at the intersection of the actual and predicted classes in the matrix
     * with the specified count.
     *
     * @param actual The actual class.
     * @param predicted The predicted class.
     * @param count The number of instances where the actual class was classified as the predicted class.
     */
    void updateMatrixValue(const QString &actual, const QString &predicted, int count);
};

#endif // CONFUSIONMATRIXWIDGET_H
