//=======================================================================
/**
 * @file FileListWidget.h
 * @author Priyonto M rahman
 * @copyright Copyright (C) 2023 Priyonto M rahman
 *
 * MIT License
 *
 * Copyright (c) 2023 Priyonto M rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================

#ifndef FILELISTWIDGET_H
#define FILELISTWIDGET_H

#include <QWidget>
#include <QVBoxLayout>
#include <QLabel>
#include <QEvent>
#include <QScrollArea>

#include "AudioSample.h"

/**
 * @ingroup TrainingUI
 * @class   FileListWidget
 * @brief   A widget for displaying a list of audio files
 *
 * FileListWidget provides a scrollable area for displaying a list of audio files,
 * and emits signals when a file is selected, or when the user interacts with
 * the "Import file" or "Clear files" buttons.
 */
class FileListWidget : public QWidget
{
    Q_OBJECT
public:
    /**
     * @brief Construct a new FileListWidget
     *
     * @param parent The parent widget
     */
    explicit FileListWidget(QWidget *parent = nullptr);

    /**
     * @brief Destroy the FileListWidget
     */
    ~FileListWidget();

    /**
     * @brief Creates the widget for the FileListWidget.
     *
     * @return A pointer to the created widget.
     */
    QWidget* createWidget();

public slots:
    /**
     * @brief Populate the file list with the given list of AudioSample files.
     *
     * @param files The list of AudioSample files.
     */
    void populateList(QList<AudioSample> files);

    /**
     * @brief Clear the file list.
     */
    void clearList();

    /**
     * @brief Get the file path of the selected file.
     *
     * @return The file path of the selected file.
     */
    QString getSelectedFilePath();

    /**
     * @brief Filters events if this object has been installed as an event filter for the watched object.
     *
     * @param obj The object being watched.
     * @param event The event to be filtered.
     * @return true if the event should be filtered out (i.e., stopped); otherwise false.
     */
    bool eventFilter(QObject *obj, QEvent *event);

signals:
    /**
     * @brief Signal emitted when an audio file is selected.
     *
     * @param filePath The file path of the selected audio file.
     */
    void didSelectAudioFile(QString);

    /**
     * @brief Signal emitted when the "Import file" button is clicked.
     */
    void didPushImportFileButton();

    /**
     * @brief Signal emitted when the "Clear files" button is clicked.
     */
    void didPushClearFileButton();

private:
    QScrollArea *scrollArea; ///< Scroll area for the file list.
    QVBoxLayout *scrollLayout; ///< Layout for the scroll area.
    QWidget* scrollAreaContentWidget; ///< Widget containing the scroll area content.

    QList<QLabel*> m_labels; ///< List of labels for the file names.
    QLabel *m_currentSelection; ///< The currently selected file.

};

#endif // FILELISTWIDGET_H
