//=======================================================================
/**
 * @file JackClientAudioEngine.h
 * @author Priyonto M rahman
 * @copyright Copyright (C) 2023 Priyonto M rahman
 *
 * MIT License
 *
 * Copyright (c) 2023 Priyonto M rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================

#ifndef JACKCLIENTAUDIOENGINE_H
#define JACKCLIENTAUDIOENGINE_H

#include <memory>

#include "jackaudioio.hpp"
#include "SourceType.h"
#include "FilterType.h"
#include "AudioFile.h"
#include "BaseFilter.h"

/**
 * @ingroup JackExternalClient
 * @class JackClientAudioEngine
 * @brief An audio engine class for handling audio processing through the JACK Audio Connection Kit.
 *
 * The JackClientAudioEngine class manages audio input and output processing, allowing for
 * filtering of audio data and control over various audio parameters.
 */
class JackClientAudioEngine: public JackCpp::AudioIO {
public:

    /**
     * @brief Constructor for JackClientAudioEngine
     *
     * Initializes a JackClientAudioEngine instance with default settings.
     */
    JackClientAudioEngine() noexcept;

    /**
     * @brief The audio processing callback function.
     *
     * Processes audio data frames through JACK, managing input and output buffers.
     *
     * @param nframes Number of frames to process.
     * @param inBufs Vector of input buffers.
     * @param outBufs Vector of output buffers.
     * @return int Status of the callback execution (0 for success).
     */
    int audioCallback(jack_nframes_t nframes,
                      audioBufVector inBufs,
                      audioBufVector outBufs) override;

    /**
     * @brief Set the volume level.
     *
     * @param level The volume level to set.
     */
    void setVolumeLevel(int level) noexcept;

    /**
     * @brief Set the filter configuration.
     *
     * @param frequency The cutoff frequency of the filter.
     * @param gain The gain of the filter in dB.
     */
    void setFilterConfig(int frequency, int gain) noexcept;

    /**
     * @brief Set the audio file for playback.
     *
     * @param file The audio file to play.
     */
    void setAudioFile(const AudioFile<float>& file);

    /**
     * @brief Set whether the audio is being modified.
     *
     * @param playingModified True if the audio is being modified, false otherwise.
     */
    void setIsPlayingModified(bool playingModified) noexcept;

    /**
     * @brief Set the audio source type.
     *
     * @param type The source type to set.
     */
    void setAudioSource(SourceType type) noexcept;

    /**
     * @brief Set the active filter type.
     *
     * @param filterType The filter type to set.
     */
    void setActiveFilterType(FilterType filterType) noexcept;

private:
    std::unique_ptr<BaseFilter> filter; ///< The filter object created with sample rate, gain, frequency and filter type.
    int sample_rate = 44100; ///< The sample rate of the audio data.
    float volume = 0.5; ///< The volume level of the audio output.
    int cut_frequency = 10000; ///< The cutoff frequency of the filter.
    int db_gain = 12; ///< The gain of the filter in dB.
    size_t samplePosition = 0; ///< The current sample position in the audio file.
    AudioFile<float> audioFile; ///< The audio file for playback.
    bool isPlayingModified = false; ///< Whether the audio is being modified.
    SourceType sourceType = SourceType::Local; ///< The source type of the audio.
    FilterType activeFilterType = FilterType::HighShelf; ///< The active filter type.
};

#endif // JACKCLIENTAUDIOENGINE_H

