//=======================================================================
/**
 * @file SessionListWidget.h
 * @author Priyonto M rahman
 * @copyright Copyright (C) 2023 Priyonto M rahman
 *
 * MIT License
 *
 * Copyright (c) 2023 Priyonto M rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================

#ifndef SESSIONLISTWIDGET_H
#define SESSIONLISTWIDGET_H

#include <QWidget>
#include <QVBoxLayout>
#include <QLabel>
#include <QEvent>
#include <QScrollArea>

#include "TestingSession.h"

/**
 * @ingroup StatisticsUI
 * @class SessionListWidget
 * @brief The SessionListWidget class provides a widget for displaying a list of TestingSession objects.
 *
 * This widget displays a scrollable list of TestingSession objects, with each session being selectable by the user.
 * It emits a signal when a session is selected.
 */
class SessionListWidget : public QWidget
{
    Q_OBJECT
public:
    /**
     * @brief Constructs a SessionListWidget with the given parent.
     * @param parent The parent widget. It defaults to nullptr.
     */
    explicit SessionListWidget(QWidget *parent = nullptr);

    /**
     * @brief Destroys the SessionListWidget.
     */
    ~SessionListWidget();

    /**
     * @brief Creates and returns a QWidget to be used as the main widget.
     * @return A pointer to the QWidget created.
     */
    QWidget* createWidget();

    /**
     * @brief Returns the session ID of the currently selected session.
     * @return The session ID, or -1 if no session is selected.
     */
    int getSelectedSessionId() const;

public slots:
    /**
     * @brief Populates the list with the given sessions.
     * @param sessions The sessions to populate the list with.
     */
    void populateList(QVector<TestingSession> sessions);

    /**
     * @brief Clears the list of all sessions.
     */
    void clearList();

    /**
     * @brief Filters events if this object has been installed as an event filter for the watched object.
     * @param obj The object being watched.
     * @param event The event that occurred.
     * @return true if further event processing should be stopped, otherwise returns false to continue event propagation.
     */
    bool eventFilter(QObject *obj, QEvent *event);

signals:
    /**
     * @brief This signal is emitted when a testing session is selected.
     * @param sessionId The ID of the selected session.
     */
    void didSelectTestingSession(int sessionId);

private:
    QScrollArea *scrollArea;
    QVBoxLayout *scrollLayout;
    QWidget* scrollAreaContentWidget;

    QList<QLabel*> m_labels;
    QLabel *m_currentSelection;

    /**
     * @brief Converts a duration from seconds to a formatted string.
     *
     * This method takes a duration in seconds and formats it into a string
     * in the format "X hr Y m Z s", where X is the number of hours, Y is the
     * number of minutes, and Z is the number of seconds.
     * If the duration is less than an hour, the hours part is omitted.
     * If the duration is less than a minute, both the hours and minutes parts are omitted.
     *
     * @param durationInSeconds The duration to format, in seconds.
     * @return A QString representing the duration in the format "X hr Y m Z s".
     */
    QString formatDuration(int durationInSeconds) const;
};

#endif // SESSIONLISTWIDGET_H

