//=======================================================================
/**
 * @file StatisticsSummaryWidget.h
 * @author Priyonto M rahman
 * @copyright Copyright (C) 2023 Priyonto M rahman
 *
 * MIT License
 *
 * Copyright (c) 2023 Priyonto M rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================

#ifndef STATISTICSUMMARYWIDGET_H
#define STATISTICSUMMARYWIDGET_H

#include <QWidget>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QLabel>

/**
 * @ingroup StatisticsUI
 * @class StatisticsSummaryWidget
 * @brief The StatisticsSummaryWidget class provides a custom QWidget to display various statistical summaries.
 *
 * This widget uses QLabel to display average trials, average response time, average duration, and average success.
 */
class StatisticsSummaryWidget : public QWidget
{
    Q_OBJECT

public:
    /**
     * @brief Constructs a new StatisticsSummaryWidget object.
     *
     * @param parent The parent QWidget. Default is nullptr.
     */
    StatisticsSummaryWidget(QWidget *parent = nullptr);

    /**
     * @brief Update the displayed average number of trials.
     *
     * @param value The new average number of trials.
     */
    void updateAverageTrials(double value);

    /**
     * @brief Update the displayed average response time.
     *
     * @param value The new average response time.
     */
    void updateAverageResponseTime(double value);

    /**
     * @brief Update the displayed average duration.
     *
     * @param value The new average duration.
     */
    void updateAverageDuration(double value);

    /**
     * @brief Update the displayed average success rate.
     *
     * @param value The new average success rate.
     */
    void updateAverageSuccess(double value);

private:
    QLabel *averageTrialsLabel; ///< The QLabel to display the average number of trials.
    QLabel *averageResponseTimeLabel; ///< The QLabel to display the average response time.
    QLabel *averageDurationLabel; ///< The QLabel to display the average duration.
    QLabel *averageSuccessLabel; ///< The QLabel to display the average success rate.

    /**
     * @brief Creates a QVBoxLayout containing a statistical summary.
     *
     * @param title The title of the summary.
     * @param valueLabel A pointer to a QLabel pointer to store the QLabel that will display the value of the statistic.
     *
     * @return A QVBoxLayout containing the statistical summary.
     */
    QVBoxLayout *createStatisticBox(const QString &title, QLabel **valueLabel);
};

#endif // STATISTICSUMMARYWIDGET_H
