//=======================================================================
/**
 * @file TrainingWindow.h
 * @author Priyonto M rahman
 * @copyright Copyright (C) 2023 Priyonto M rahman
 *
 * MIT License
 *
 * Copyright (c) 2023 Priyonto M rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================

#ifndef TRAININGWINDOW_H
#define TRAININGWINDOW_H

#include <QComboBox>
#include <QEvent>
#include <QFocusEvent>
#include <QPushButton>
#include <QSlider>
#include <QStackedWidget>
#include <QThread>
#include <QWidget>
#include <QTimer>

#include "BaseFilter.h"
#include "CustomComboBox.h"
#include "DatabaseManager.h"
#include "EQGraph.h"
#include "FileListWidget.h"
#include "FilterType.h"
#include "JackWorker.h"
#include "SessionType.h"

/**
 * @ingroup TrainingUI
 * @class TrainingWindow
 * @brief Provides a widget for conducting training sessions.
 *
 * This class is responsible for managing and displaying the training window.
 * It includes methods for creating widgets, populating audio files, and managing
 * the training session state, among others.
 */
class TrainingWindow : public QWidget
{
    Q_OBJECT
public:
    explicit TrainingWindow(QWidget *parent = nullptr); ///< Constructs a TrainingWindow with an optional parent widget.
    ~TrainingWindow(); ///< Destructor.

    QWidget* createWidget(); ///< Creates the widget for the training window.
    QWidget* createLandingWidget(); ///< Creates the landing widget for the training window.
    QWidget* createTrainingWidget(); ///< Creates the training widget.

    void showTrainingWidget(); ///< Shows the training widget.
    void showLandingWidget(); ///< Shows the landing widget.

    void setDatabaseManager(DatabaseManager* manager); ///< Sets the database manager.
    void populateAudioFiles(); ///< Populates the list of audio files.

signals:
    void didChangeTrainingSessionState(SessionType sessionType); ///< Signal emitted when the training session state changes.
    void didChangeSliderValue(int gain, int cutOffFrequency); ///< Signal emitted when the slider value changes.
    void didChangeAudioFile(QString filePath); ///< Signal emitted when the audio file changes.
    void didChangeAudioFilter(FilterType filter); ///< Signal emitted when the audio filter changes.

private:
    // UI
    QPushButton* startStopButton;
    QTimer *cpuLoadTimer;
    QLabel* cpuLoadLabel;
    QSlider* volumeSlider;
    QSlider* gainSlider;
    QSlider* cutOfFrequencySlider;
    EQGraph* eqGraph;

    QStackedWidget widgetContainer;
    QWidget* trainingWidget;
    QWidget* landingWidget;

    FileListWidget* fileListWidget;
    QWidget* fileListContentWidget;

    CustomComboBox *sourceSelectionDropdown;
    CustomComboBox *filterSelectionDropdown;

    // Threading
    QThread* workerThread;
    JackWorker* jackWorker;

    // Database
    DatabaseManager* dbManager;


    // Others
    AudioFile<float> currentAudioFile;
    FilterType activeFilterType = FilterType::HighShelf;
    std::shared_ptr<BaseFilter> activeFilter;
    bool isAudioFileValid = false;

private slots:
    void sliderValueDidChange(int gain, int cutOffFrequency); ///< Handles the event of slider value change.
    void didSelectAudioFile(QString filePath); ///< Handles the event of an audio file selection.
    void didSelectAudioFilter(FilterType filter); ///< Handles the event of an audio filter selection.
    void showErrorDialog(const QString &message); ///< Shows an error dialog.
};

#endif // TRAININGWINDOW_H

