//=======================================================================
/**
 * @file BinomialHelper.cpp
 * @author Priyonto M Rahman
 * @copyright Copyright (c) 2023 Priyonto M Rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================

#include "BinomialHelper.h"

double BinomialHelper::logFactorial(int n) {
    // Calculate the natural logarithm of the factorial of n.
    double result = 0.0;
    for (int i = 1; i <= n; ++i) {
        result += std::log(i);
    }
    return result;
}

double BinomialHelper::logCombination(int n, int k) {
    // Calculate the natural logarithm of the combination of n items taken k at a time.
    return logFactorial(n) - (logFactorial(k) + logFactorial(n - k));
}

int BinomialHelper::combination(int n, int k) {
    // Calculate the combination of n items taken k at a time.
    return std::round(std::exp(logCombination(n, k)));
}

double BinomialHelper::binomialProbability(int n, int k, double p) {
    // Calculate the binomial probability of getting k successes in n trials.
    return combination(n, k) * std::pow(p, k) * std::pow(1 - p, n - k);
}

double BinomialHelper::binomialCDF(int n, int k, double p) {
    // Calculate the cumulative probability of getting up to k successes in n trials.
    double cdf = 0.0;
    for (int i = 0; i <= k; ++i) {
        cdf += binomialProbability(n, i, p);
    }
    return cdf;
}

int BinomialHelper::inverseBinomialCDF(double targetP, int n, double p) {
    // Find the smallest number of successes such that the CDF is greater than or equal to targetP.
    int k = 0;
    double cdf = 0.0;
    while (cdf < targetP && k <= n) {
        cdf = binomialCDF(n, k, p);
        if (cdf < targetP) {
            k++;
        }
    }
    return k;
}
