//=======================================================================
/**
 * @file ConfusionMatrixWidget.cpp
 * @author Priyonto M Rahman
 * @copyright Copyright (c) 2023, Priyonto M Rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================


#include "ConfusionMatrixWidget.h"
#include "QtWidgets/qheaderview.h"

ConfusionMatrixWidget::ConfusionMatrixWidget(QWidget *parent)
    : QWidget(parent)
{
    matrixTable = new QTableWidget(2, 2, this);
    QStringList hHeaders = {"H0 is True (inaudible)\nPredicted X=A", "H0 is False (audible)\nPredicted X=B"};
    QStringList vHeaders = {"Do no Reject H0 (inaudible)\nActual X=A", "Reject H0 (audible)\nActual X=B"};
    matrixTable->setHorizontalHeaderLabels(hHeaders);
    matrixTable->setVerticalHeaderLabels(vHeaders);
    matrixTable->setWordWrap(true);

    // Disable cell selection and editing
    matrixTable->setSelectionMode(QAbstractItemView::NoSelection);
    matrixTable->setEditTriggers(QAbstractItemView::NoEditTriggers);

    matrixTable->horizontalHeader()->setDefaultSectionSize(150);
    matrixTable->verticalHeader()->setDefaultSectionSize(150);

    // Initialize the matrix items
    for(int i = 0; i < 2; ++i)
    {
        for(int j = 0; j < 2; ++j)
        {
            QTableWidgetItem* item = new QTableWidgetItem("0");
            item->setTextAlignment(Qt::AlignCenter);  // Center align the text
            matrixTable->setItem(i, j, item);
        }
    }

    matrixTable->setSizeAdjustPolicy(QTableWidget::AdjustToContents);

}

void ConfusionMatrixWidget::updateMatrixValue(const QString &actual, const QString &predicted, int count)
{
    int row = actual == "A" ? 0 : 1;
    int col = predicted == "A" ? 0 : 1;

    QString equation;
    if (actual == predicted)
    {
        equation = predicted == "A" ? "Correct \n 1- ⍺" : "Correct \n 1 - β \n (power)";
    }
    else
    {
        if(actual == "A" && predicted == "B")
        {
            equation = "Type II Error β";
        }
        else if(actual == "B" && predicted == "A")
        {
            equation = "Type I Error ⍺";
        }
    }

    // Add both count and equation to the cell
    matrixTable->item(row, col)->setText(QString::number(count) + "\n" + equation);

    // if actual and predicted are same, set background color to green; else red
    if (actual == predicted)
    {
        matrixTable->item(row, col)->setBackground(QBrush(Qt::darkGreen));
        matrixTable->item(row, col)->setForeground(QBrush(Qt::white));
    }
    else
    {
        matrixTable->item(row, col)->setBackground(QBrush(Qt::darkRed));
        matrixTable->item(row, col)->setForeground(QBrush(Qt::white));
    }
}


void ConfusionMatrixWidget::populateMatrix(QMap<QString, QMap<QString, int>> &confusionMatrixCounts)
{
    for (const auto &actual : confusionMatrixCounts.keys()) {
        for (const auto &predicted : confusionMatrixCounts[actual].keys()) {
            int count = confusionMatrixCounts[actual][predicted];
            updateMatrixValue(actual, predicted, count);
        }
    }
}
