//=======================================================================
/**
 * @file EQGraph.cpp
 * @author Priyonto M rahman
 * @copyright Copyright (C) 2023 Priyonto M rahman
 *
 * MIT License
 *
 * Copyright (c) 2023 Priyonto M rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================

#include "EQGraph.h"

EQGraph::EQGraph(QWidget *parent)
    : QWidget(parent), m_axisX(nullptr), m_axisY(nullptr)
{
    // Set up the chart and series
    m_chart = new QChart();
    m_series = new QLineSeries();
    m_chart->addSeries(m_series);
    m_chart->setAnimationDuration(5);

    // Set up the chart view and add the chart to it
    m_chartView = new QChartView(m_chart);
    m_chartView->setRenderHint(QPainter::Antialiasing);
    QVBoxLayout *layout = new QVBoxLayout(this);
    layout->addWidget(m_chartView);
}

EQGraph::~EQGraph()
{
    delete m_axisX;
    delete m_axisY;
    delete m_chartView;
    delete m_chart;
    delete m_series;
}

void EQGraph::plot(const std::vector<double>& x,
                   const std::vector<double>& y,
                   int gainRange,
                   const QString& title,
                   const QString& xAxisLabel,
                   const QString& yAxisLabel)
{
    // Ensure the sizes of x and y are the same
    if (x.size() != y.size())
    {
        qDebug() << "Error: x and y must have the same size";
        return;
    }

    // Clear the series
    m_series->clear();

    // Add the data to the series
    for (size_t i = 0; i < x.size(); i++)
    {
        m_series->append(x[i], y[i]);
    }

    // Set the chart title and axis labels
    m_chart->setTitle(title);

    if (m_axisX != nullptr)
    {
        m_chart->removeAxis(m_axisX);
        delete m_axisX;
    }

    if (m_axisY != nullptr)
    {
        m_chart->removeAxis(m_axisY);
        delete m_axisY;
    }

    m_axisY = new QValueAxis();
    m_axisY->setTitleText(yAxisLabel);
    m_axisY->setRange(-gainRange, gainRange);
    m_chart->addAxis(m_axisY, Qt::AlignLeft);
    m_series->attachAxis(m_axisY);

    m_axisX = new QLogValueAxis();
    m_axisX->setTitleText(xAxisLabel);
    m_axisX->setLabelFormat("%g");
    m_axisX->setBase(10.0);
    m_axisX->setMinorTickCount(-1);

    // Dynamic range setting according to data
    auto [minElement, maxElement] = std::minmax_element(x.begin(), x.end());
    m_axisX->setRange(*minElement, *maxElement);

    m_chart->addAxis(m_axisX, Qt::AlignBottom);
    m_series->attachAxis(m_axisX);
}
