//=======================================================================
/**
 * @file JackClientAudioEngine.cpp
 * @author Priyonto M rahman
 * @copyright Copyright (C) 2023 Priyonto M rahman
 *
 * MIT License
 *
 * Copyright (c) 2023 Priyonto M rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================

#include "JackClientAudioEngine.h"

#include <memory>

#include "BaseFilter.h"
#include "ShelfFilter.h"

JackClientAudioEngine::JackClientAudioEngine() noexcept
    : JackCpp::AudioIO("JACK External Client", 2, 2),
    samplePosition(0) {
    // Create filter with default values.
    filter = std::make_unique<ShelfFilter>(sample_rate, cut_frequency, db_gain, activeFilterType);
}

int JackClientAudioEngine::audioCallback(jack_nframes_t nframes,
                                         audioBufVector inBufs,
                                         audioBufVector outBufs) {

    // Process audio from the correct source
    if (sourceType == SourceType::JackInput) {
        for (unsigned int channel = 0; channel < inBufs.size(); ++channel) {
            for (unsigned int i = 0; i < nframes; ++i) {
                float sample = isPlayingModified ? filter->process_sample(inBufs[channel][i]) : inBufs[channel][i];
                outBufs[channel][i] = sample * volume;
            }
        }
    } else {
        // Considering stereo audio.
        if (audioFile.samples.size() < 2) {
            return 0;
        }

        jack_default_audio_sample_t *out1 = outBufs[0];
        jack_default_audio_sample_t *out2 = outBufs[1];

        // Process each sample one at a time
        for (jack_nframes_t i = 0; i < nframes; ++i) {
            jack_default_audio_sample_t sample1 = audioFile.samples[0][samplePosition];
            jack_default_audio_sample_t sample2 = audioFile.samples[1][samplePosition];

            // Apply the filter if necessary
            if (isPlayingModified) {
                out1[i] = filter->process_sample(sample1) * volume;
                out2[i] = filter->process_sample(sample2) * volume;
            } else {
                out1[i] = sample1 * volume;
                out2[i] = sample2 * volume;
            }

            // Move to the next sample position, looping if necessary
            samplePosition++;
            if (samplePosition >= audioFile.getNumSamplesPerChannel()) {
                samplePosition = 0;
            }
        }
    }

    return 0;
}

void JackClientAudioEngine::setVolumeLevel(int level) noexcept {
    volume = static_cast<float>(level) * 0.01f;
}

void JackClientAudioEngine::setFilterConfig(int frequency, int gain) noexcept {
    db_gain = gain;
    cut_frequency = frequency;
    // Create  new filter object with updated settings.
    // TODO: Improve this.
    filter = std::make_unique<ShelfFilter>(sample_rate, cut_frequency, db_gain, activeFilterType);
}

void JackClientAudioEngine::setAudioFile(const AudioFile<float>& file) {
    audioFile = file;
    samplePosition = 0;
}

void JackClientAudioEngine::setIsPlayingModified(bool playingModified) noexcept {
    isPlayingModified = playingModified;
}

void JackClientAudioEngine::setAudioSource(SourceType type) noexcept {
    sourceType = type;
}

void JackClientAudioEngine::setActiveFilterType(FilterType filterType) noexcept {
    activeFilterType = filterType;
    // Create  new filter object with updated settings.
    // TODO: Improve this.
    filter = std::make_unique<ShelfFilter>(sample_rate, cut_frequency, db_gain, activeFilterType);
}
